/********************************************************/
/* File name :	as.h					*/
/*							*/
/* Include this header file to your application program.*/
/* This file consists of constants and subroutines to	*/
/* control CN2000  'ASPP' ports.                        */
/*							*/
/* Each physical 'ASPP' port is divided into two        */
/* logical ports, one is 'command port', the other      */
/* is 'data port'. The 'command port' is used to set    */
/* port baud, parity, etc. The 'data port' is used to   */
/* transfer real data.					*/
/********************************************************/

/********************************************/
/*	CN2000	Constants definition	    */
/********************************************/
#define D_START_DATAPORT	950  /* 950: TCP port no for 1st data port    */
				     /* 951: TCP port no for 2nd data port    */
				     /* 952: TCP port no for 3rd data port    */
				     /* 953: TCP port no for 4th data port    */
				     /* ....				      */
				     /* 965: TCP port no for 16th data port   */

#define D_START_CMDPORT 	966  /* 966: TCP port no for 1st cmd port     */
				     /* 967: TCP port no for 2nd cmd port     */
				     /* 968: TCP port no for 3rd cmd port     */
				     /* ....				      */
				     /* 981: TCP port no for 16th cmd port    */

#define D_COMMAND_IOCTL 	16	/* IOCTL command	       */
#define D_COMMAND_IOCTL_EX 	64	/* IOCTL command	       */
#define D_COMMAND_FLOWCTRL	17	/* Flow control command        */
#define D_COMMAND_LINECTRL	18	/* Line status control command */
#define D_COMMAND_LSTATUS	19	/* Line status read command    */
#define D_COMMAND_FLUSH 	20	/* Buffer flush command        */
#define D_COMMAND_IQUEUE	21	/* Input queue command	       */
#define D_COMMAND_OQUEUE	22	/* Output queue command        */
#define D_COMMAND_PORT_RESET	32	/* Port reset command	       */

/* parameters for D_COMMAND_IOCTL ioctl command */
#define D_IOCTL_B300		0	/* IOCTL : baud rate = 300 bps	 */
#define D_IOCTL_B600		1	/* IOCTL : baud rate = 600 bps	 */
#define D_IOCTL_B1200		2	/* IOCTL : baud rate = 1200 bps  */
#define D_IOCTL_B2400		3	/* IOCTL : baud rate = 2400 bps  */
#define D_IOCTL_B4800		4	/* IOCTL : baud rate = 4800 bps  */
#define D_IOCTL_B7200		5	/* IOCTL : baud rate = 7200 bps  */
#define D_IOCTL_B9600		6	/* IOCTL : baud rate = 9600 bps  */
#define D_IOCTL_B19200		7	/* IOCTL : baud rate = 19200 bps */
#define D_IOCTL_B38400		8	/* IOCTL : baud rate = 38400 bps */
#define D_IOCTL_B57600		9	/* IOCTL : baud rate = 57600 bps */
#define D_IOCTL_B115200 	10	/* IOCTL : baud rate = 115200 bps */
#define D_IOCTL_B230400 	11	/* IOCTL : baud rate = 230400 bps */
#define D_IOCTL_B460800 	12	/* IOCTL : baud rate = 460800 bps */

#define D_IOCTL_BIT8		3	/* IOCTL : 8 data bits */
#define D_IOCTL_BIT7		2	/* IOCTL : 7 data bits */
#define D_IOCTL_BIT6		1	/* IOCTL : 6 data bits */
#define D_IOCTL_BIT5		0	/* IOCTL : 5 data bits */

#define D_IOCTL_STOP1		0	/* IOCTL : 1 stop bit  */
#define D_IOCTL_STOP2		4	/* IOCTL : 2 stop bits */

#define D_IOCTL_EVEN		8	/* IOCTL : even parity */
#define D_IOCTL_ODD		16	/* IOCTL : odd parity  */
#define D_IOCTL_NONE		0	/* IOCTL : none parity */

/* parameters for D_COMMAND_FLOWCTRL flow control command */
#define D_FCTRL_CTS_YES 	1	/* yes, do CTS H/W flow control      */
#define D_FCTRL_CTS_NO		0	/* no, don't do CTS H/W flow control */
#define D_FCTRL_RTS_YES 	1	/* yes, do RTS H/W flow control      */
#define D_FCTRL_RTS_NO		0	/* no, don't do RTS H/W flow control */
#define D_FCTRL_TXXON_YES	1	/* yes, Tx XON/XOFF S/W flow control */
#define D_FCTRL_TXXON_NO	0	/* no Tx XON/XOFF S/W flow control   */
#define D_FCTRL_RXXON_YES	1	/* yes, Rx XON/XOFF S/W flow control */
#define D_FCTRL_RXXON_NO	0	/* no Rx XON/XOFF S/W flow control   */

/* parameters for D_COMMAND_LINECTRL line control command	*/
#define D_LCTRL_DTR_ON		1	/* set DTR on		*/
#define D_LCTRL_DTR_OFF 	0	/* set DTR off		*/
#define D_LCTRL_RTS_ON		1	/* set RTS on		*/
#define D_LCTRL_RTS_OFF 	0	/* set RTS off		*/

/* parameters for D_COMMAND_FLUSH flush buffer command		*/
#define D_FLUSH_RXBUFFER	0	/* flush Rx buffer	*/
#define D_FLUSH_TXBUFFER	1	/* flush Tx buffer	*/
#define D_FLUSH_ALLBUFFER	2	/* flush Rx & Tx buffer */

/* parameters for D_COMMAND_LSTATUS line status command */
#define D_LSTATUS_DSR_ON	1	/* line status : DSR on 	*/
#define D_LSTATUS_DSR_OFF	0	/* line status : DSR off	*/
#define D_LSTATUS_CTS_ON	1	/* line status : CTS on 	*/
#define D_LSTATUS_CTS_OFF	0	/* line status : CTS off	*/
#define D_LSTATUS_DCD_ON	1	/* line status : DCD on 	*/
#define D_LSTATUS_DCD_OFF	0	/* line status : DCD off	*/

#define 	CN2000_MAXPORT		16
#define 	FAIL			0xFFFFFFFF
#define 	OK			0

/* Total timeout = RETRY * INTERVAL = 5000 msec */
#define 	INTERVAL		100	    /* 100 msec  */
#define 	RETRY			50

extern	errno;


int WaitAck(SOCKET fd_cmd,char cmd);
SOCKET sioopen(unsigned long ipaddr,unsigned short p);
/*************************************************************/
/*	Useful subroutines to control 'ASPP' port

	1. sio_copen(ipaddr, p)
	   Open command port.
	2. sio_dopen(ipaddr, p)
	   Open data port.
	3. sio_reset(fd_cmd)
	   Reset a port.
	4. sio_ioctl(fd_cmd, baud, mode)
	   Set port baud, parity, etc.
	5. sio_flowctrl(fd_cmd, CTSmode, RTSmode, TxXon, RxXon)
	   Set hardware and/or software flow control.
	6. sio_lctrl(fd_cmd, DTRon, RTSon)
	   Line control.
	7. sio_flush(fd_cmd, mode)
	   Flush input/output buffer.
	8. sio_lstatus(fd_cmd)
	   Check line status.
	9. sio_iqueue(fd_cmd)
	   Check how many data in input buffer.
	10.sio_oqueue(fd_cmd)
	   Check how many data still in output buffer.

******************************************************************/

/*---------------------------------------------------------------*/
/* 1. create socket for command port				 */
/* 2. connect to that command port				 */
/*    Note: command port is used for control purpose,		 */
/*    e.g. set async port baud rate, read its line status...	 */
/*---------------------------------------------------------------*/
SOCKET sio_copen(ipaddr, p)
unsigned long	ipaddr;
int		p;
{
	unsigned short	cmd_port;


	if ( p < 1 || p > CN2000_MAXPORT ) return(INVALID_SOCKET);
	cmd_port = D_START_CMDPORT + p - 1;
	return(sioopen(ipaddr,cmd_port));
}

/*---------------------------------------------------------------*/
/* 1. create socket for data port				 */
/* 2. connect to that data port 				 */
/*    Note: data port is used for read/write real data. 	 */
/*---------------------------------------------------------------*/
SOCKET	 sio_dopen(ipaddr, p)
unsigned long	ipaddr;
int		p;
{
	unsigned short		data_port;

	if ( p < 1 || p > CN2000_MAXPORT ) return(FAIL);
	data_port = D_START_DATAPORT + p - 1;
	return(sioopen(ipaddr,data_port));
}

/* set up the port configuration, e.g. baud rate, via command port */
int sio_ioctl(fd_cmd, baud, mode)
SOCKET		fd_cmd;
unsigned char	baud, mode;
{
	unsigned char		buf[20];

	buf[0] = D_COMMAND_IOCTL;
	buf[1] = 2;
	buf[2] = baud;
	buf[3] = mode;
	send(fd_cmd, buf, 4, 0);
	return( WaitAck(fd_cmd, buf[0]) );
}

/* set up the port configuration, and save settings */
/* in flash rom, e.g. baud rate, via command port */
int sio_ioctl_ex(fd_cmd, baud, mode)
SOCKET		fd_cmd;
unsigned char	baud, mode;
{
	unsigned char		buf[20];

	buf[0] = D_COMMAND_IOCTL_EX;
	buf[1] = 2;
	buf[2] = baud;
	buf[3] = mode;
	send(fd_cmd, buf, 4, 0);
	return( WaitAck(fd_cmd, buf[0]) );
}

/* set up flow control via command port */
int sio_flowctrl(fd_cmd, CTSmode, RTSmode, TxXon, RxXon)
SOCKET		fd_cmd;
unsigned char	CTSmode, RTSmode, TxXon, RxXon;
{
	unsigned char		buf[20];

	buf[0] = D_COMMAND_FLOWCTRL;
	buf[1] = 4;
	buf[2] = CTSmode;
	buf[3] = RTSmode;
	buf[4] = TxXon;
	buf[5] = RxXon;
	send(fd_cmd, buf, 6, 0);
	return( WaitAck(fd_cmd, buf[0]) );
}

int sio_reset(fd_cmd)
SOCKET		fd_cmd;
{
	unsigned char		buf[20];

	buf[0] = D_COMMAND_PORT_RESET;
	buf[1] = 3;
	buf[2] = 0x31;
	buf[3] = 0x36;
	buf[4] = 0x61;
	send(fd_cmd, buf, 5, 0);
	return( WaitAck(fd_cmd, buf[0]) );
}

/* set DTR, RTS on via command port */
int sio_lctrl(fd_cmd, DTRon, RTSon)
SOCKET		fd_cmd;
unsigned char	DTRon, RTSon;
{
	unsigned char		buf[20];

	buf[0] = D_COMMAND_LINECTRL;
	buf[1] = 2;
	buf[2] = DTRon;
	buf[3] = RTSon;
	send(fd_cmd, buf, 4, 0);
	return( WaitAck(fd_cmd, buf[0]) );
}

/* flush Async Server input/output buffer */
int sio_flush(fd_cmd, mode)
SOCKET		fd_cmd;
unsigned char	mode;
{
	unsigned char		buf[20];

	buf[0] = D_COMMAND_FLUSH;
	buf[1] = 1;
	buf[2] = mode;
	send(fd_cmd, buf, 3, 0);
	return( WaitAck(fd_cmd, buf[0]) );
}

/* get port status */
int sio_lstatus(fd_cmd)
SOCKET		fd_cmd;
{
/*
return = stat :   bit no  ...6 5 4 3 2 1 0
				     ^ ^
				     | |___ CTS
				     |_____ DCD
*/


	unsigned char	buf[20], tmp[1];
	int		i,j,step,stat;

	buf[0] = D_COMMAND_LSTATUS;
	buf[1] = 0;
	send(fd_cmd, buf, 2, 0);
	step = stat = 0;
	j = 0;
	while ( 1 ) {
		if( (i=recv(fd_cmd, tmp, 1,0)) == 1){
			switch(step){
			   case(0):
				if(tmp[0]==buf[0]) step = 1;
			   case(1):
				if(tmp[0]==3) step = 2;
				else return(FAIL);
			   case(2):
				step = 3;
			   case(3):
				if(tmp[0]==D_LSTATUS_CTS_ON) stat |= 2;
				step = 4;
			   case(4):
				if(tmp[0]==D_LSTATUS_DCD_ON) stat |= 4;
				return(stat);
			}
		} else {
		    Sleep(INTERVAL);
		    if (j++ >= RETRY)
			return(FAIL);
		}
	}
}

/* get port input buffer size */
int sio_iqueue(fd_cmd)
SOCKET		fd_cmd;
{
	unsigned char	buf[20];
	int		i,j;

	buf[0] = D_COMMAND_IQUEUE;
	buf[1] = 0;
	send(fd_cmd, buf, 2, 0);
	j = 0;
	for (;;) {
	    Sleep(INTERVAL);
	    i = recv(fd_cmd, buf, 4, 0);
	    if (i == 4) {
		if ( buf[0] == D_COMMAND_IQUEUE && buf[1] == 2 ) {
			i = *(int *)(&buf[2]) & 0x0000ffff;
			return(i);
		}
	    }
	    if (j++ >= RETRY)
		break;
	}
	return(FAIL);
}

/* get output buffer size */
int sio_oqueue(fd_cmd)
SOCKET		fd_cmd;
{
	unsigned char	buf[20];
	int		i,j;

	buf[0] = D_COMMAND_OQUEUE;
	buf[1] = 0;
	send(fd_cmd, buf, 2, 0);
	j = 0;
	for (;;) {
	    Sleep(INTERVAL);
	    i = recv(fd_cmd, buf, 4, 0);
	    if (i == 4) {
		if ( buf[0] == D_COMMAND_OQUEUE && buf[1] == 2 ) {
			i = *(int *)(&buf[2]) & 0x0000ffff;
			return(i);
		}
	    }
	    if (j++ >= RETRY)
		break;
	}
	return(FAIL);
}

int WaitAck(fd_cmd, cmd)
SOCKET fd_cmd;
char cmd;
{
int	i, j, n, step;
char	tmp[1];

	n = step = j = 0;
	while ( 1 ) { /* wait for 3 bytes: cmd, 'O', 'K' */
		if( (i=recv(fd_cmd, tmp, 1, 0)) == 1){
			switch(step){
			   case(0):
				if(tmp[0]==cmd) step = 1;
				break;
			   case(1):
				if(tmp[0]=='O') step = 2;
				else return(FAIL);
				break;
			   case(2):
				if(tmp[0]=='K') return(OK);
				else return(FAIL);
				break;
			}
		} else {
		    Sleep(INTERVAL);
		    if (j++ >= RETRY)
			return(FAIL);
		}
	}
}

SOCKET sioopen(ipaddr, p)
unsigned long	ipaddr;
unsigned short	p;
{
	struct sockaddr_in	des;
	int			i,j,len;
	SOCKET			fd;
	BOOL			b = TRUE;
	ULONG			mode = 1;   /* set to non_delay mode */


	p = htons(p);
	fd = socket(AF_INET, SOCK_STREAM, 0);
	if ( fd == INVALID_SOCKET ) {
	    return(fd);
	}
	if (ioctlsocket(fd,FIONBIO,&mode)) {
	    closesocket(fd);
	    return(INVALID_SOCKET);
	}
	des.sin_family = AF_INET;
	des.sin_addr.s_addr = ipaddr;
	des.sin_port = p;
	len = sizeof(struct sockaddr_in);

	/*-------------------------*/
	/* connect to aspp port */
	/*-------------------------*/
	i = 0;
	for (;;) {
		j = connect(fd,(struct sockaddr *)&des, len);
		if (j == 0)
		    break;
		if (WSAGetLastError() == WSAEISCONN) {
			j = 0;
			break;
		}
		Sleep(INTERVAL);
		if (i++ >= RETRY)
		    break;
	}
	if( j != 0 ) {
	    closesocket(fd);
	    return(INVALID_SOCKET);
	}
	return(fd);
}
